"""
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
"""
from flask import json, request

from accountant import session_scope

from ..models.scheduled_operations import ScheduledOperation

from .. import api


@api.route("/scheduled_operations/<account_id>")
def get_scheduled_operations(account_id):
    """
    Return entries for an account, year, and month.
    """
    with session_scope() as session:
        query = ScheduledOperation.get_scheduled_operations_for_account(
            session, account_id)

        return json.dumps([{
            "id": i.id,
            "start_date": i.start_date.strftime("%Y-%m-%d"),
            "stop_date": i.stop_date.strftime("%Y-%m-%d"),
            "day": str(i.day),
            "frequency": str(i.frequency),
            "label": i.label,
            "value": str(i.value),
            "category": i.category,
            "account_id": i.account_id
        } for i in query.all()])


@api.route("/scheduled_operations", methods=["PUT"])
def add_scheduled_operation():
    with session_scope() as session:
        scheduledOperation = ScheduledOperation(
            start_date=request.json['start_date'],
            stop_date=request.json['stop_date'],
            day=request.json['day'],
            frequency=request.json['frequency'],
            label=request.json['label'],
            value=request.json['value'],
            category=request.json['category'],
            account_id=request.json['account_id']
        )

        session.add(scheduledOperation)

        return json.dumps("Scheduled operation added.")


@api.route("/scheduled_operations/<scheduled_operation_id>", methods=["PUT"])
def update_scheduled_operation(scheduled_operation_id):
    with session_scope() as session:
        query = session.query(ScheduledOperation)
        query = query.filter(ScheduledOperation.id == scheduled_operation_id)

        scheduledOperation = query.first()

        scheduledOperation.id = scheduled_operation_id
        scheduledOperation.start_date = request.json['start_date'],
        scheduledOperation.stop_date = request.json['stop_date'],
        scheduledOperation.day = request.json['day'],
        scheduledOperation.frequency = request.json['frequency'],
        scheduledOperation.label = request.json['label']
        scheduledOperation.value = request.json['value']
        scheduledOperation.category = request.json['category']
        scheduledOperation.account_id = request.json['account_id']

        session.merge(scheduledOperation)

        return json.dumps(
            "Scheduled operation #%s updated." % scheduled_operation_id)


@api.route("/scheduled_operations/<scheduled_operation_id>", methods=["DELETE"])
def delete_scheduled_operation(scheduled_operation_id):
    with session_scope() as session:
        query = session.query(ScheduledOperation)
        query = query.filter(ScheduledOperation.id == scheduled_operation_id)
        scheduledOperation = query.first()

        session.delete(scheduledOperation)

        return json.dumps(
            "Scheduled operation #%s deleted." % scheduled_operation_id)
